; 
    include \masm32\include\masm32rt.inc
; 

comment * -----------------------------------------------------
                        Build this  template with
                       "CONSOLE ASSEMBLE AND LINK"

        The "get_ml" procedure functions as a "get next statement"
        procedure for assembler and similar source code.
        It handles split lines of code by determining the last
        non white space character before any comments on the same
        line. It sets a split line if the last character is either
        a comma "," or a "\" character. If the line continuation
        character is "\" it is overwritten by the next character in
        the output.

        The procedure supports text in double quotes which allows
        the quoted text to contain characters like ",\;" without
        triggering the line continuation or comment handing code.
        If the quoted text does not have a closing quote, a -1
        error is returned in EAX which is determined by either
        an ascii 10 or ascii ZERO.

        The procedure is designed to be used with a variable
        that would normally be set to zero and it returns the same
        variable with the location of the next read position in the
        source code.

        If the procedure finds the end of a buffer with an ascii ZERO
        it returns zero in EAX.

        The output is trimmed at the start and has CRLF pairs
        removed as well as comments but no attempt has been
        made to format spacing between the start and end of
        each statement as most tokeniser designs are capable
        of distinguishing each argument without any further
        formatting.

        ----------------------------------------------------- *

    get_ml PROTO :DWORD,:DWORD,:DWORD

    .code

start:
   
; 

    call main
    ;; inkey
    exit

; 

main proc

    LOCAL hMem  :DWORD
    LOCAL hBuf  :DWORD
    LOCAL flen  :DWORD
    LOCAL rpos  :DWORD

    mov hMem, InputFile("getml.asm")
    mov flen, ecx
    mov hBuf, alloc(flen)

    mov rpos, 0
  @@:
    mov rpos, rv(get_ml,hMem,hBuf,rpos)
    print hBuf,13,10
    cmp rpos, 0
    jne @B

    free hBuf
    free hMem

    ret

main endp

; 

align 4

get_ml proc src:DWORD,dst:DWORD,rpos:DWORD

    LOCAL lchar     :DWORD              ; last non white space character offset

    push ebx
    push esi
    push edi

    mov lchar, 0
    mov esi, src
    add esi, rpos
    mov edi, dst
    mov ecx, -1

  ; ****************************************************
  ; ftrim loop scans through spaces, tabs, comments and
  ; unused lines to find the start of the next statement
  ; ****************************************************
  align 4
  ftrim:
    add ecx, 1
    cmp BYTE PTR [esi+ecx], 32
    je ftrim
    cmp BYTE PTR [esi+ecx], 9
    je ftrim
    cmp BYTE PTR [esi+ecx], 13
    je ftrim
    cmp BYTE PTR [esi+ecx], 10
    je ftrim
    cmp BYTE PTR [esi+ecx], ";"
    je decomnt
    sub ecx, 1

  ; ********************************************
  ; main loop copies the statement to the buffer
  ; ********************************************
  align 4
  lpst:
    add ecx, 1
    mov al, [esi+ecx]                   ; read BYTE
    cmp al, 13                          ; CR
    je lpst
    cmp al, 10                          ; LF
    je cmntout                          ; test if line continuator is set
    cmp al, ";"                         ; comment char
    je commnt
    cmp al, 34
    je dblquote
    mov [edi], al                       ; write BYTE
    add edi, 1
    test al, al                         ; 0 terminator
    jz quit
    cmp al, 32                          ; if space
    je lpst
    cmp al, 9                           ; or tab, loop back
  lastchar:
    je lpst
    mov lchar, ecx                      ; store the last char position
    jmp lpst

  align 4
  decomnt:
    add ecx, 1
    cmp BYTE PTR [esi+ecx], 10
    je ftrim
    cmp BYTE PTR [esi+ecx], 0
    je quit
    jmp decomnt

  align 4
  commnt:
    add ecx, 1
    cmp BYTE PTR [esi+ecx], 10
    je cmntout
    cmp BYTE PTR [esi+ecx], 0
    je quit
    jmp commnt

  ; *****************************************
  ; evaluate the end of line condition to see
  ; if the line is a split line of code.
  ; *****************************************
  align 4
  cmntout:
    mov eax, lchar
    cmp BYTE PTR [esi+eax], 44          ; ","
    je ftrim
    cmp BYTE PTR [esi+eax], 92          ; "\"
    jne setrv
    sub edi, 1                          ; overwrite "\" with the next write
    jmp lpst

  ; *********************
  ; process double quotes
  ; *********************
  align 4
  dblquote:
    mov [edi], al                       ; write BYTE
    add edi, 1
  @@:
    add ecx, 1
    mov al, [esi+ecx]                   ; read BYTE
    mov [edi], al                       ; write BYTE
    add edi, 1
    test al, al
    jz qterror
    cmp al, 10
    jz qterror
    cmp al, 34
    jne @B
    mov lchar, ecx                      ; store the last char position
    add lchar, 1
    jmp lpst

  qterror:
    mov eax, -1
    jmp @F

  quit:                                 ; zero terminator exit
    mov BYTE PTR [edi], 0
    xor eax, eax                        ; return ZERO at end of data
    jmp @F

  setrv:
    mov BYTE PTR [edi], 0
    mov eax, rpos
    add eax, ecx                        ; set the next read position

  @@:
    pop edi
    pop esi
    pop ebx

    ret

get_ml endp

; 

end start
















